﻿using HeuristicLab.Clients.Hive.WebJobManager.Services;
using HeuristicLab.Common;
using Microsoft.AspNet.Http;
using Microsoft.AspNet.SignalR;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace HeuristicLab.Clients.Hive.WebJobManager
{
    /// <summary>
    /// SignalR Hub for updating a current selected job
    /// </summary>
    public class JobUpdaterHub : Hub
    {
        
        private RefreshableJob Job;//Current job (only used as reference, instance gets lost)

        private WebLoginService weblog;
        private FileOpeningService fileopener;
        private Guid userId;

        private void loader()
        {
            weblog = WebLoginService.Instance;
            string uid = Context.QueryString["userid"];
            if (uid == null || uid == "" || Guid.Parse(uid) == Guid.Empty)
            {
                userId = Guid.Empty;
            }
            else {
                userId = Guid.Parse(uid);
                fileopener = weblog.getFileOpener(userId);
            }
        }
        /// <summary>
        /// Initial connection call from client
        /// </summary>
        public void initConnection()
        {
            loader();
            fileopener.previousids = new List<Guid>();
            fileopener.previousLogs = new List<int>();
            Job = fileopener.Job;
            updateAll();//start initial update

        }
        /// <summary>
        /// Starts update loop for selected job. Sends end token when done and calls GC
        /// </summary>
        public void updateAll()
        {
            loader();
            fileopener.refreshJob();//refresh all data from job
            Job = fileopener.Job;
            updateJob();
            foreach (var t in Job.HiveTasks)
            {
                looperTasks(t);//starts recursive loop
            }
            Clients.Caller.requestDone();//Final note from server when all tasks are checked.
            //Client starts countdown and resends the updateAll when done
            GC.Collect();
            GC.WaitForPendingFinalizers();

        }
        private void updateJob()
        {
            Clients.Caller.processJobData(Job.Job.CalculatingCount, Job.Job.FinishedCount);
        }
        /// <summary>
        /// Recursive loop for updating the selected job view
        /// </summary>
        /// <param name="task">Loops on all children</param>
        private void looperTasks(HiveTask task)
        {
            try
            {
                int index;
                bool test = false;
                if (fileopener.previousids.Contains(task.Task.Id))
                {
                    index = fileopener.previousids.IndexOf(task.Task.Id);
                }
                else
                {//initial add to previous list, used to check if updates happened
                    fileopener.previousids.Add(task.Task.Id);
                    index = fileopener.previousids.IndexOf(task.Task.Id);
                    fileopener.previousLogs.Add(task.Task.StateLog.Count);
                    test = true;//initial added task, data must be sent
                }
                var previous = fileopener.previousLogs[index];
                if (test || previous < task.Task.StateLog.Count)
                {//Checks if change happened so data is not sent unnecessary 
                    fileopener.previousLogs[index] = task.Task.StateLog.Count;
                    JsonSerializerSettings settings = new JsonSerializerSettings();
                    settings.ContractResolver = new JsonTaskResolver();
                    //limits loaded data 
                    var json = JsonConvert.SerializeObject(task.Task, settings);
                    Clients.Caller.processData(task.Task.Id, json, task.ItemTask.Name.ToString());
                    //data sent to user
                }
                foreach (var t in task.ChildHiveTasks)
                {//loop on childs
                    looperTasks(t);
                }
            }
            catch (JsonSerializationException e) { }//Taskresolver fixes much but img tries to throw this sometimes
        }
        /// <summary>
        /// Restarts a failed task, sent by client
        /// </summary>
        /// <param name="id">Task id from failed task</param>
        public void restartTask(Guid id)
        {
            loader();
            weblog.getServiceLocator(userId).getHiveServiceClient().RestartTask(id);
        }
    }
}
