﻿using HeuristicLab.Clients.Hive.WebJobManager.ViewModels;
using Microsoft.AspNet.Hosting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

namespace HeuristicLab.Clients.Hive.WebJobManager.Services
{
    /// <summary>
    /// Used for opening a file and uploading this to Hive.
    /// Saves all relevant Job information needed for uploading.
    /// Outsourced to selectedJob keep the current job loaded
    /// </summary>
    public class FileOpeningService
    {
        public Guid UserId { get; set; }

        private WebLoginService weblog;
        public RefreshableJob Job { get; set; }
        public FileOpeningViewModel vm { get; set; }
        public IHostingEnvironment env { get; set; }
        public List<Guid> previousids {
            get; set; }
        public List<int> previousLogs { get; set; }
        

        public FileOpeningService(Guid u)
        {
            weblog = WebLoginService.Instance;
            UserId = u;
        }


        /// <summary>
        /// Clears loaded jobs and start new one
        /// </summary>
        public void NewModel()
        {
            vm = new FileOpeningViewModel();
            Job = new RefreshableJob();
            Job.Job.Name = "init";
            
        }
        /// <summary>
        /// Attaches tasks from viewmodel to job
        /// </summary>
        public void setTasks()
        {
            
            Job.HiveTasks.Add(vm.SelectedTask);
        }
        /// <summary>
        /// Refresh all info from a job from the server
        /// </summary>
        public void refreshJob()
        {
            weblog.getClientWeb(UserId).LoadJob(Job);
        }
        /// <summary>
        /// Adds loaded job from vm onto server
        /// </summary>
        /// <returns>Job being uploaded</returns>
        public RefreshableJob AddCurrentModelToHive()
        {
            if (vm != null)
            {
                var clientWeb = weblog.getClientWeb(UserId);
                clientWeb.CurrentEnv = env;
                clientWeb.Refresh();
                clientWeb.StartJob((ex) => { throw ex; }, Job, CancellationToken.None);
                return Job;
            }
            return null;
        }
    }
}
